@@ -67,15 +67,15 @@ enum Mode { m_none, m_append, m_concatenate, m_create, m_extract, m_list };
 
 void show_help( const long num_online )
   {
-  std::printf( "Tarlz is a combined implementation of the tar archiver and the lzip\n"
-               "compressor. By default tarlz creates, lists and extracts archives in a\n"
-               "simplified posix pax format compressed with lzip on a per file basis. Each\n"
-               "tar member is compressed in its own lzip member, as well as the end-of-file\n"
-               "blocks. This method adds an indexed lzip layer on top of the tar archive,\n"
-               "making it possible to decode the archive safely in parallel. The resulting\n"
-               "multimember tar.lz archive is fully backward compatible with standard tar\n"
-               "tools like GNU tar, which treat it like any other tar.lz archive. Tarlz can\n"
-               "append files to the end of such compressed archives.\n"
+  std::printf( "Tarlz is a massively parallel (multi-threaded) combined implementation of\n"
+               "the tar archiver and the lzip compressor. Tarlz creates, lists and extracts\n"
+               "archives in a simplified posix pax format compressed with lzip, keeping the\n"
+               "alignment between tar members and lzip members. This method adds an indexed\n"
+               "lzip layer on top of the tar archive, making it possible to decode the\n"
+               "archive safely in parallel. The resulting multimember tar.lz archive is\n"
+               "fully backward compatible with standard tar tools like GNU tar, which treat\n"
+               "it like any other tar.lz archive. Tarlz can append files to the end of such\n"
+               "compressed archives.\n"
                "\nThe tarlz file format is a safe posix-style backup format. In case of\n"
                "corruption, tarlz can extract all the undamaged members from the tar.lz\n"
                "archive, skipping over the damaged members, just like the standard\n"
@@ -87,10 +87,11 @@ void show_help( const long num_online )
                "  -h, --help                 display this help and exit\n"
                "  -V, --version              output version information and exit\n"
                "  -A, --concatenate          append tar.lz archives to the end of an archive\n"
+               "  -B, --data-size=<bytes>    set target size of input data blocks [2x8=16 MiB]\n"
                "  -c, --create               create a new archive\n"
                "  -C, --directory=<dir>      change to directory <dir>\n"
                "  -f, --file=<archive>       use archive file <archive>\n"
-               "  -n, --threads=<n>          set number of decompression threads [%ld]\n"
+               "  -n, --threads=<n>          set number of (de)compression threads [%ld]\n"
                "  -q, --quiet                suppress all messages\n"
                "  -r, --append               append files to the end of an archive\n"
                "  -t, --list                 list the contents of an archive\n"
@@ -98,8 +99,9 @@ void show_help( const long num_online )
                "  -x, --extract              extract files from an archive\n"
                "  -0 .. -9                   set compression level [default 6]\n"
                "      --asolid               create solidly compressed appendable archive\n"
-               "      --dsolid               create per-directory compressed archive\n"
-               "      --no-solid             create per-file compressed archive (default)\n"
+               "      --bsolid               create per block compressed archive (default)\n"
+               "      --dsolid               create per directory compressed archive\n"
+               "      --no-solid             create per file compressed archive\n"
                "      --solid                create solidly compressed archive\n"
                "      --anonymous            equivalent to '--owner=root --group=root'\n"
                "      --owner=<owner>        use <owner> name/ID for files added\n"
@@ -237,6 +239,20 @@ int open_outstream( const std::string & name, const bool create )
   }
 
 
+// This can be called from any thread, main thread or sub-threads alike,
+// since they all call common helper functions that call cleanup_and_fail()
+// in case of an error.
+//
+void cleanup_and_fail( const int retval )
+  {
+  // only one thread can delete and exit
+  static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+
+  pthread_mutex_lock( &mutex );		// ignore errors to avoid loop
+  std::exit( retval );
+  }
+
+
 void show_error( const char * const msg, const int errcode, const bool help )
   {
   if( verbosity < 0 ) return;
@@ -284,8 +300,8 @@ int main( const int argc, const char * const argv[] )
     { show_error( "Bad library version. At least lzlib 1.0 is required." );
       return 1; }
 
-  enum { opt_ano = 256, opt_aso, opt_crc, opt_dbg, opt_dso, opt_grp, opt_kd,
-         opt_nso, opt_own, opt_per, opt_sol, opt_un };
+  enum { opt_ano = 256, opt_aso, opt_bso, opt_crc, opt_dbg, opt_dso, opt_grp,
+         opt_kd, opt_nso, opt_own, opt_per, opt_sol, opt_un };
   const Arg_parser::Option options[] =
     {
     { '0',  0,                 Arg_parser::no  },
@@ -299,6 +315,7 @@ int main( const int argc, const char * const argv[] )
     { '8',  0,                 Arg_parser::no  },
     { '9',  0,                 Arg_parser::no  },
     { 'A', "concatenate",      Arg_parser::no  },
+    { 'B', "data-size",        Arg_parser::yes },
     { 'c', "create",           Arg_parser::no  },
     { 'C', "directory",        Arg_parser::yes },
     { 'f', "file",             Arg_parser::yes },
@@ -313,6 +330,7 @@ int main( const int argc, const char * const argv[] )
     { 'x', "extract",          Arg_parser::no  },
     { opt_ano, "anonymous",    Arg_parser::no  },
     { opt_aso, "asolid",       Arg_parser::no  },
+    { opt_bso, "bsolid",       Arg_parser::no  },
     { opt_dbg, "debug",        Arg_parser::yes },
     { opt_dso, "dsolid",       Arg_parser::no  },
     { opt_grp, "group",        Arg_parser::yes },
@@ -338,7 +356,8 @@ int main( const int argc, const char * const argv[] )
   for( int argind = 0; argind < parser.arguments(); ++argind )
     {
     const int code = parser.code( argind );
-    if( !code ) { ++filenames; continue; }		// skip non-options
+    if( !code )						// skip non-options
+      { if( parser.argument( argind ).size() ) ++filenames; continue; }
     const std::string & sarg = parser.argument( argind );
     const char * const arg = sarg.c_str();
     switch( code )
@@ -347,6 +366,8 @@ int main( const int argc, const char * const argv[] )
       case '5': case '6': case '7': case '8': case '9':
                 level = code - '0'; break;
       case 'A': set_mode( program_mode, m_concatenate ); break;
+      case 'B': cl_data_size = getnum( arg, min_data_size, max_data_size );
+                break;
       case 'c': set_mode( program_mode, m_create ); break;
       case 'C': break;					// skip chdir
       case 'f': if( sarg != "-" ) archive_name = sarg; break;
@@ -361,6 +382,7 @@ int main( const int argc, const char * const argv[] )
       case 'x': set_mode( program_mode, m_extract ); break;
       case opt_ano: set_owner( "root" ); set_group( "root" ); break;
       case opt_aso: solidity = asolid; break;
+      case opt_bso: solidity = bsolid; break;
       case opt_crc: missing_crc = true; break;
       case opt_dbg: debug_level = getnum( arg, 0, 3 ); break;
       case opt_dso: solidity = dsolid; break;
@@ -387,7 +409,7 @@ int main( const int argc, const char * const argv[] )
     case m_none:   show_error( "Missing operation.", 0, true ); return 2;
     case m_append:
     case m_create: return encode( archive_name, parser, filenames, level,
-                                  program_mode == m_append );
+                          num_workers, debug_level, program_mode == m_append );
     case m_concatenate: return concatenate( archive_name, parser, filenames );
     case m_extract:
     case m_list:   return decode( archive_name, parser, filenames, num_workers,
